import { Observable } from 'iterable-observer';
export declare enum BodyRequestMethods {
    POST = "POST",
    PUT = "PUT",
    PATCH = "PATCH",
    DELETE = "DELETE"
}
export interface RequestOptions {
    withCredentials?: boolean;
    timeout?: number;
    responseType?: XMLHttpRequestResponseType;
}
export interface Request extends RequestOptions {
    method?: 'HEAD' | 'GET' | keyof typeof BodyRequestMethods;
    path: string | URL;
    headers?: HeadersInit;
    body?: BodyInit | HTMLFormElement | any;
}
export interface Response<B = Request['body']> {
    status: number;
    statusText: string;
    headers: {
        [key: string]: string | object;
    };
    body?: B;
}
export declare class HTTPError<B = Request['body']> extends URIError {
    status: number;
    statusText: string;
    headers: {
        [key: string]: string | object;
    };
    body?: B;
    constructor(message: string, response: Response);
}
export interface LinkHeader {
    [rel: string]: {
        URI: string;
        rel: string;
        title?: string;
    };
}
export declare const headerParser: {
    Link(value: string): LinkHeader;
};
export declare function parseHeaders(raw: string): Response['headers'];
export declare function request<B>({ method, path, headers, body, ...rest }: Request): {
    response: Promise<Response<B>>;
    upload: Observable<ProgressEvent<EventTarget>>;
    download: Observable<ProgressEvent<EventTarget>>;
};
